% [figureHandles,figureHandleAll]=ECE_plot_10({'sys1',{tar_llrs1,nontar_llrs1}},{'sys2',{tar_llrs2,nontar_llrs2}},...)
%
% ECE_plot_10 plots an Empirical-Cross-Entropy plot (ECE plot).
%     Three sets of LR values are represented:
%       - Red solid curve: experimental set of log-LR values
%         (NATURAL LOGARITHM).
%       - Blue dashed curve: experimental set of log-LR values
%         transformed by Pool Adjacent Violators (PAV) 
%         algorithm. This supposedly gives a best-calibrated
%         reference for the given set of log-LR values.
%       - Neutral method (log-LR=0 always).
%
% IMPORTANT:
% For clarity, the x-axis are the logarithm of the prior odds
% in base-10. HOWEVER, LOG-LR VALUES SHOULD BE INTRODUCED TO
% THE FUNCTION AS NATURAL LOGARITHMS
%
% Every value of ECE is considered as a function of the prior
% probabilities. 
%
% -- INPUT --
%      * One or more cell arrays, each one for each set of log-LR values
%            under analysis. Format: {nameTest1,{tar_llrs1,nontar_llrs1}}.
%              -- nameTest1: a string label for the experimental set of
%                 log-LR values.
%              -- tar_llrs1: an row vector with the same-source (target) 
%                 log-LR values in the experimental set (NATURAL LOG-LR).
%              -- nontar_llrs1: an row vector with the different-source
%                 (non-target) log-LR values in the experimental set
%                 (NATURAL LOG-LR).
% 
% -- OUTPUT --
%      * figureHandles: a vector containing the figure handles (one
%        per set of log-LR values).
%      * figureHandleAll: the handle to a figure containing all the 
%        plots generated.
%
% Author: Daniel Ramos-Castro, October 2007
%
% Based on the work of Niko Brummer (http://www.dsp.sun.ac.za/~nbrummer/). See also:
%   Niko Brummer and Johan du Preez, "Application-Independent Evaluation of Speaker Detection"
%   Computer Speech and Language, to be published, 2005. 
%
% The code in this function is free, and can be freely distributed. However,
% if its you use it for any publication, please include
% the following reference in the final published work:
% 
% - D. Ramos, 2007. "Forensic Evaluation of the Evidence Using Automatic 
%   Speaker Recognition Systems." Ph.D. Thesis. Departamento de Ingenieria
%   Informatica. Universidad Autonoma de Madrid.
% 
% Available at:
% http://atvs.ii.uam.es/files/2007_11_28_thesis_daniel_ramos_preprint_v1.pdf

function [figureHandles,figureHandleAll]=ECE_plot_10(varargin);

% number of systems
n=length(varargin); 

% plo = prior_log_odds
% This range was chosen to show more or less the range of interest for current levels of
% ECE performance. If systems get better and evaluation data more, this range 
% can be enlarged.
% The theoretcal range is the whole real line.
plo=-2.5:0.1:2.5;

pointsPriors=length(plo);

clog=zeros(pointsPriors,n);
minclog=zeros(pointsPriors,n);
priorEntropy=zeros(pointsPriors,n);

% Optimizing LRs for each input trial
for i=1:n;

  data = varargin{i}; 
  name = data{1};
  llrs = data{2};
  scores = llrs;
  
  % LR optimization
  [tar_opt_llrs,nontar_opt_llrs] = opt_loglr(scores{1},scores{2},'raw');
  
  % Computation of ECE for each value of the prior probability
  for contPriors=1:pointsPriors,
    clog(contPriors,i) = cllr_prior(llrs{1},llrs{2},sigmoid_10(plo(contPriors)));
    minclog(contPriors,i) = cllr_prior(tar_opt_llrs,nontar_opt_llrs,sigmoid_10(plo(contPriors)));
    priorEntropy(contPriors,i) = -(sigmoid_10(plo(contPriors))*log(sigmoid_10(plo(contPriors))) + (1-sigmoid_10(plo(contPriors)))*log(1-sigmoid_10(plo(contPriors))))/log(2);
  end,
  
end;

maxClog=max(max(clog));

figureHandles=zeros(1,n)-1;

if n>1,
    figureHandleAll=figure;
    axesHandleSubPlot=subplot(1,n,1);
    axesAll=get(figureHandleAll,'currentAxes');
    set(axesAll,'FontSize',14);
    %Size of the figure depends on the number of plots
    figureDimensions=get(0,'ScreenSize');
    figureDimensions(3)=min(figureDimensions(3),figureDimensions(3)*n/2);
    figureDimensions(4)=figureDimensions(4)*0.8;
    set(figureHandleAll,'Position',figureDimensions);
    hold on;
end;

% Drawing. One figure for each LR set (system)
for i=1:n
    % Plotting each separate plot
    figureHandles(1,i)=figure;
    hold on;
    a=get(figureHandles(1,i),'currentAxes');
    set(a,'FontSize',14);
    
    colormap([0,0.7,0;1,0,0]);

    plot(plo,clog(:,i),'r','LineWidth',3);
    plot(plo,minclog(:,i),'b--','LineWidth',3);
    plot(plo,priorEntropy(:,i),'k:','LineWidth',3);

    xlabel('Prior log_{10}(odds)');
    ylabel('Empirical cross-entropy');
       
    axis([min(plo),max(plo),0,min(maxClog,1)]); 
    legend('LR values','After PAV','LR=1 always');
    data=varargin{i};
    title(data{1});
    grid;
    plot([0 0],[0 1000],'k--','LineWidth',2);
    
    if n>1,
        % Plotting in the composite plot
        figure(figureHandleAll);
        axesHandleSubPlot=subplot(1,n,i);
        axesAll=get(figureHandleAll,'currentAxes');
        set(axesAll,'FontSize',14);
        hold on;

        colormap([0,0.7,0;1,0,0]);

        plot(plo,clog(:,i),'r','LineWidth',3);
        plot(plo,minclog(:,i),'b--','LineWidth',3);
        plot(plo,priorEntropy(:,i),'k:','LineWidth',3);

        xlabel('Prior log_{10}(odds)');
        ylabel('Empirical cross-entropy');

        axis([min(plo),max(plo),0,min(maxClog,1)]); 
        data=varargin{i};
        title(data{1});
        grid;
        plot([0 0],[0 1000],'k--','LineWidth',2);
    end;
end,

if n>1,
    legend('LR values','After PAV','LR=1 always','Location','NorthEast');
end;
