% ECE_plot_individual_contribution_10({'sys1',{tar_llrs1,nontar_llrs1}},{'sys2',{tar_llrs2,nontar_llrs2}},...)
% ECE_plot_individual_contribution_10({'sys1',{tar_llrs1,nontar_llrs1}},{'sys2',{tar_llrs2,nontar_llrs2}},...,nContributors)
%
% ECE_plot_individual_contribution_10 plots the individual 
% contribution to Emprirical Cross-Entropy (ECE) of the target
% (same-source) and non-target (differnt-source) log-LR values
% separately.
%
% IMPORTANT:
% For clarity, the x-axis are the logarithm of the prior odds
% in base-10. HOWEVER, LOG-LR VALUES SHOULD BE INTRODUCED TO
% THE FUNCTION AS NATURAL LOGARITHMS
%
% Every value of ECE is considered as a function of the prior
% probabilities. 
%
% -- INPUT --
%      * One or more cell arrays, each one for each set of log-LR values
%            under analysis. Format: {nameTest1,{tar_llrs1,nontar_llrs1}}.
%              -- nameTest1: a string label for the experimental set of
%                 log-LR values.
%              -- tar_llrs1: an row vector with the same-source (target) 
%                 log-LR values in the experimental set (NATURAL LOG-LR).
%              -- nontar_llrs1: an row vector with the different-source
%                 (non-target) log-LR values in the experimental set
%                 (NATURAL LOG-LR).
%      * nContributors: if especified, indicates the number of target and
%            non-target "worst" log-LR values to be plot. This avoids too
%            high computational burden when computing the plots.
% 
% -- OUTPUT --
%      * figureHandlesTarget: a vector containing the figure handles (one
%        per set of log-LR values) for same-source (target) log-LR values.
%      * figureHandlesNonTarget: a vector containing the figure handles (one
%        per set of log-LR values) for different-source (non-target) log-LR
%        values.
%
% Author: Daniel Ramos-Castro, October 2007
%
% Based on the work of Niko Brummer (http://www.dsp.sun.ac.za/~nbrummer/). See also:
%   Niko Brummer and Johan du Preez, "Application-Independent Evaluation of Speaker Detection"
%   Computer Speech and Language, to be published, 2005. 
%
% The code in this function is free, and can be freely distributed. However,
% if its you use it for any publication, please include
% the following reference in the final published work:
% 
% - D. Ramos, 2007. "Forensic Evaluation of the Evidence Using Automatic 
%   Speaker Recognition Systems." Ph.D. Thesis. Departamento de Ingenieria
%   Informatica. Universidad Autonoma de Madrid.
% 
% Available at:
% http://atvs.ii.uam.es/files/2007_11_28_thesis_daniel_ramos_preprint_v1.pdf

function [figureHandlesTarget,figureHandlesNonTarget]=ECE_plot_individual_contribution_10(varargin);

nSystems=length(varargin);

if ~iscell(varargin{nSystems}),
    nContributors=varargin{nSystems};
    nSystems=nSystems-1;
end;

figureHandlesTarget=zeros(1,nSystems);
figureHandlesNonTarget=zeros(1,nSystems);

% plo = prior_log_odds
% This range was chosen to show more or less the range of interest for current levels of
% speaker detection performance. If systems get better and evaluation data more, this range 
% can be enlarged.
% The theoretical range is the whole real line. However, the empirical
% nature of ECE makes it non-statistically-significant as long as errors
% start to be scarce, i.e., as long as the prior log-odds goes to extreme
% values.
plo=-2.5:0.1:2.5;

pointsPriors=length(plo);

% Optimizing LRs for each input trial
for i=1:nSystems;

  data = varargin{i}; 
  name = data{1};
  llrs = data{2};
  % Target scores are ordered from worst to best
  scoresTarget = sort(llrs{1});
  % Non-Target scores are ordered from worst to best
  scoresNonTarget = sort(llrs{2},'descend');
  if ~iscell(varargin{length(varargin)}),
    nScoresTarget=min(length(scoresTarget),nContributors);
    nScoresNonTarget=min(length(scoresNonTarget),nContributors);
  else
    nScoresTarget=length(scoresTarget);
    nScoresNonTarget=length(scoresNonTarget);
  end;
  
  clogTarget=zeros(pointsPriors,nScoresTarget);
  clogNonTarget=zeros(pointsPriors,nScoresNonTarget);
  
  % Computation of ECE and ECE+D for each prior
  for contPriors=1:pointsPriors,
      for contScoresTarget=1:nScoresTarget,  
        clogTarget(contPriors,contScoresTarget) = cllr_prior(scoresTarget(contScoresTarget),[],sigmoid_10(plo(contPriors)));
      end;
      for contScoresNonTarget=1:nScoresNonTarget,  
        clogNonTarget(contPriors,contScoresNonTarget) = cllr_prior([],scoresNonTarget(contScoresNonTarget),sigmoid_10(plo(contPriors)));
      end;
  end,
  
  maxY=max(max(max(clogTarget)),max(max(clogNonTarget)));
  
  figureHandlesTarget(i)=figure;
  hold on;
  a=get(figureHandlesTarget(i),'currentAxes');
  set(a,'FontSize',14);
  colormap([0,0.7,0;1,0,0]);

  for contScoresTarget=1:nScoresTarget,
    plot(plo,clogTarget(:,contScoresTarget),'r','LineWidth',1);
  end;
 
  xlabel('Prior log_{10}(odds)');
  ylabel(['ECE for each same-source LR value * N_s_s']);

  axis([min(plo),max(plo),0,maxY]); 
  title([data{1} ' same-source']);
  grid;
  plot([0 0],[0 maxY],'k--','LineWidth',2);

  figureHandlesNonTarget(i)=figure;
  hold on;
  a=get(figureHandlesNonTarget(i),'currentAxes');
  set(a,'FontSize',14);
  colormap([0,0.7,0;1,0,0]);

  for contScoresNonTarget=1:nScoresNonTarget,
    plot(plo,clogNonTarget(:,contScoresNonTarget),'r','LineWidth',1);
  end;
 
  xlabel('Prior log_{10}(odds)');
  ylabel(['ECE for each different-source LR value * N_d_s']);

  axis([min(plo),max(plo),0,maxY]); 
  title([data{1} ' different-source']);
  grid;
  plot([0 0],[0 maxY],'k--','LineWidth',2);
  
end;
